{% load media %}
<!doctype html>

<head>
    {% block meta %}
        <meta charset=utf-8>
        <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
        <meta name="viewport" content="width=device-width,initial-scale=1">
    {% endblock %}

    <title>{% block title %}{{ page_title }}{% endblock %}</title>
    {% block favicon %}{% favicon url='favicon.png?'|add:codename %}{% endblock %}

    {% block bootstrap %}
        {% stylesheet url='bootstrap/dist/css/bootstrap.min.css' %}
    {% endblock %}

    {% block era %}
        {% stylesheet url='styles/era.css' %}
        {% script url='scripts/era.js' %}
    {% endblock %}

    {% block bower %}
        {% script url='moment/min/moment-with-locales.min.js' %}
        {% script url='jquery/dist/jquery.min.js' %}
        {% script url='jquery-query-object/jquery.query-object.js' %}
        {% script url='underscore/underscore-min.js' %}
        {% script url='bootstrap/dist/js/bootstrap.min.js' %}
        {% stylesheet url='fontawesome/css/font-awesome.min.css' %}
    {% endblock %}

    {% block media %}
        <script>var django = {jQuery: $}</script>
        {{ media|safe }}
    {% endblock %}

    {% block design %}{% endblock %}
    {% block compatibility %}
        <!--[if lt IE 9]>
            <script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
            <script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
        <![endif]-->
    {% endblock %}
</head>
<body class="{{ view.about }}">
    {% block body %}{% endblock %}
</body>
